@@ -26,6 +26,9 @@
 #define _XFDESKTOP_COMMON_H_
 
 #include <glib.h>
+#include <gdk/gdk.h>
+
+#include <X11/Xlib.h>
 
 #define BACKDROP_CHANNEL         "BACKDROP"
 #define DEFAULT_BACKDROP         DATADIR "/xfce4/backdrops/xfce-smoke.png"
@@ -33,27 +36,20 @@
 #define XFDESKTOP_SELECTION_FMT  "XFDESKTOP_SELECTION_%d"
 #define XFDESKTOP_IMAGE_FILE_FMT "XFDESKTOP_IMAGE_FILE_%d"
 
+#define RELOAD_MESSAGE     "reload"
+#define MENU_MESSAGE       "menu"
+#define WINDOWLIST_MESSAGE "windowlist"
+#define QUIT_MESSAGE       "quit"
+
 G_BEGIN_DECLS
 
-typedef enum
-{
-	XFCE_BACKDROP_IMAGE_AUTO = 0,
-	XFCE_BACKDROP_IMAGE_CENTERED,
-	XFCE_BACKDROP_IMAGE_TILED,
-	XFCE_BACKDROP_IMAGE_STRETCHED,
-	XFCE_BACKDROP_IMAGE_SCALED
-} XfceBackdropImageStyle;
-
-typedef enum
-{
-	XFCE_BACKDROP_COLOR_SOLID = 0,
-	XFCE_BACKDROP_COLOR_HORIZ_GRADIENT,
-	XFCE_BACKDROP_COLOR_VERT_GRADIENT
-} XfceBackdropColorStyle;
-
 gchar **get_list_from_file(const gchar *);
 gboolean is_backdrop_list(const gchar *path);
 gboolean xfdesktop_check_image_file(const gchar *filename);
+gchar *xfce_desktop_get_menufile();
+gboolean xfdesktop_check_is_running(Window *xid);
+void xfdesktop_send_client_message(Window xid, const gchar *msg);
+gboolean xfdesktop_popup_grab_available(GdkWindow *win, guint32 timestamp);
 
 G_END_DECLS
 
