@@ -49,6 +49,10 @@
 #include <libxfce4mcs/mcs-client.h>
 #include <libxfcegui4/libxfcegui4.h>
 
+#ifdef HAVE_THUNAR_VFS
+#include <dbus/dbus-glib.h>
+#endif
+
 #include "main.h"
 #include "xfdesktop-common.h"
 #include "xfce-backdrop.h"
@@ -62,13 +66,11 @@ static SessionClient *client_session = NULL;
 static gboolean is_session_managed = FALSE;
 static gboolean desktop_gtk_menu_images = TRUE;
 
-G_MODULE_EXPORT void
-quit(gboolean force)
+static void
+session_logout()
 {
-    if(is_session_managed)
-        logout_session(client_session);
-    else
-        gtk_main_quit();
+    g_return_if_fail(is_session_managed);
+    logout_session(client_session);
 }
 
 static void
@@ -237,10 +239,21 @@ main(int argc, char **argv)
     Window xid;
     McsClient *mcs_client;
     GtkSettings *settings;
+    const gchar *message = NULL;
+    gboolean already_running;
     
+    /* bind gettext textdomain */
+    xfce_textdomain(GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
+    
+#if defined(ENABLE_DESKTOP_ICONS) && defined(HAVE_THUNAR_VFS)
+    g_thread_init(NULL);
+    dbus_g_thread_init();
+#endif
+    gtk_init(&argc, &argv);
+        
     if(argc > 1 && (!strcmp(argv[1], "--version") || !strcmp(argv[1], "-V"))) {
-        g_print(_("This is %s version %s, running on Xfce %s.\n"), PACKAGE, VERSION,
-                xfce_version_string());
+        g_print(_("This is %s version %s, running on Xfce %s.\n"), PACKAGE,
+                VERSION, xfce_version_string());
         g_print(_("Built with GTK+ %d.%d.%d, linked with GTK+ %d.%d.%d."),
                 GTK_MAJOR_VERSION,GTK_MINOR_VERSION, GTK_MICRO_VERSION,
                 gtk_major_version, gtk_minor_version, gtk_micro_version);
@@ -258,44 +271,62 @@ main(int argc, char **argv)
                 _("enabled")
 #else
                 _("disabled")
+#endif
+                );
+        g_print(_("    Desktop File Icons:  %s\n"),
+#ifdef HAVE_THUNAR_VFS
+                _("enabled")
+#else
+                _("disabled")
 #endif
                 );
         
-        exit(0);
+        return 0;
     }
 
-    /* bind gettext textdomain */
-    xfce_textdomain(GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
-    
-#if defined(ENABLE_DESKTOP_ICONS) && defined(HAVE_THUNAR_VFS)
-    /* move to xfce-desktop.c? */
-    g_thread_init(NULL);
-#endif
-    gtk_init(&argc, &argv);
-
-    if(xfdesktop_check_is_running(&xid)) {
-        DBG("xfdesktop is running");
+    if(argc > 1) {
+        const gchar *argument = argv[1];
         
-        if(argc <= 1 || strcmp("-reload", argv[1]) == 0)
-            xfdesktop_send_client_message(xid, RELOAD_MESSAGE);
-        else if(strcmp("-menu", argv[1]) == 0)
-            xfdesktop_send_client_message(xid, MENU_MESSAGE);
-        else if(strcmp("-windowlist", argv[1]) == 0)
-            xfdesktop_send_client_message(xid, WINDOWLIST_MESSAGE);
-        else if(strcmp("-quit", argv[1]) == 0)
-            xfdesktop_send_client_message(xid, QUIT_MESSAGE);
-        else {
+        /* allow both --(option) and -(option) */
+        if('-' == argument[0] && '-' == argument[1])
+            ++argument;
+        
+        if(!strcmp("-reload", argument))
+            message = RELOAD_MESSAGE;
+        else if(!strcmp("-menu", argument))
+            message = MENU_MESSAGE;
+        else if(!strcmp("-windowlist", argument))
+            message = WINDOWLIST_MESSAGE;
+        else if(!strcmp("-quit", argument))
+            message = QUIT_MESSAGE;
+        else if(!strcmp("--sm-client-id", argv[1])) {
+            /* do nothing */
+        } else {
             g_printerr(_("%s: Unknown option: %s\n"), PACKAGE, argv[1]);
             g_printerr(_("Options are:\n"));
-            g_printerr(_("    -reload      Reload all settings, refresh image list\n"));
-            g_printerr(_("    -menu        Pop up the menu (at the current mouse position)\n"));
-            g_printerr(_("    -windowlist  Pop up the window list (at the current mouse position)\n"));
-            g_printerr(_("    -quit        Cause xfdesktop to quit\n"));
+            g_printerr(_("    --reload      Reload all settings, refresh image list\n"));
+            g_printerr(_("    --menu        Pop up the menu (at the current mouse position)\n"));
+            g_printerr(_("    --windowlist  Pop up the window list (at the current mouse position)\n"));
+            g_printerr(_("    --quit        Cause xfdesktop to quit\n"));
             
             return 1;
         }
-        
-        return 0;
+    }
+    
+    already_running = xfdesktop_check_is_running(&xid);
+    if(already_running) {
+        DBG("xfdesktop is running");
+        if(!message)
+            message = RELOAD_MESSAGE;
+    }
+    
+    if(message) {
+        if(!already_running)
+            g_printerr(_("%s is not running.\n"), PACKAGE);
+        else
+            xfdesktop_send_client_message(xid, message);
+
+        return (already_running ? 0 : 1);
     }
     
     gdpy = gdk_display_get_default();
@@ -338,6 +369,12 @@ main(int argc, char **argv)
     client_session->die = session_die;
     is_session_managed = session_init(client_session);
     
+    if(is_session_managed) {
+        for(i = 0; i < nscreens; ++i)
+            xfce_desktop_set_session_logout_func(XFCE_DESKTOP(desktops[i]),
+                                                 session_logout);
+    }
+    
     menu_init(mcs_client);
     menu_set_show_icons(desktop_gtk_menu_images);
     
