@@ -66,6 +66,7 @@
 #include "xfdesktop-window-icon-manager.h"
 # ifdef HAVE_THUNAR_VFS
 # include "xfdesktop-file-icon-manager.h"
+# include "xfdesktop-special-file-icon.h"
 # endif
 #endif
 
@@ -88,6 +89,8 @@ struct _XfceDesktopPriv
     
     gboolean xinerama_stretch;
     
+    SessionLogoutFunc session_logout_func;
+    
 #ifdef ENABLE_DESKTOP_ICONS
     XfceDesktopIconStyle icons_style;
     gboolean icons_use_system_font;
@@ -107,6 +110,11 @@ static void xfce_desktop_unrealize(GtkWidget *widget);
 
 static gboolean xfce_desktop_expose(GtkWidget *w,
                                     GdkEventExpose *evt);
+static gboolean xfce_desktop_delete_event(GtkWidget *w,
+                                          GdkEventAny *evt);
+static void style_set_cb(GtkWidget *w,
+                         GtkStyle *old_style,
+                         gpointer user_data);
 
 /* private functions */
 
@@ -147,11 +155,6 @@ xfce_desktop_setup_icon_view(XfceDesktop *desktop)
 {
     XfdesktopIconViewManager *manager = NULL;
     
-    if(desktop->priv->icon_view) {
-        gtk_widget_destroy(desktop->priv->icon_view);
-        desktop->priv->icon_view = NULL;
-    }
-    
     switch(desktop->priv->icons_style) {
         case XFCE_DESKTOP_ICON_STYLE_NONE:
             /* nada */
@@ -164,20 +167,29 @@ xfce_desktop_setup_icon_view(XfceDesktop *desktop)
 #ifdef HAVE_THUNAR_VFS
         case XFCE_DESKTOP_ICON_STYLE_FILES:
             {
+                ThunarVfsPath *path;
                 gchar *desktop_path = xfce_get_homefile("Desktop",
                                                         NULL);
-                thunar_vfs_init();
-                ThunarVfsPath *path = thunar_vfs_path_new(desktop_path, NULL);
+                
+                path = thunar_vfs_path_new(desktop_path, NULL);
                 if(path) {
                     manager = xfdesktop_file_icon_manager_new(path);
-                    /* FIXME: make a pref */
+                    thunar_vfs_path_unref(path);
+                    /* FIXME: make prefs */
                     xfdesktop_file_icon_manager_set_show_removable_media(XFDESKTOP_FILE_ICON_MANAGER(manager),
                                                                          TRUE);
-                    thunar_vfs_path_unref(path);
+                    xfdesktop_file_icon_manager_set_show_special_file(XFDESKTOP_FILE_ICON_MANAGER(manager),
+                                                                      XFDESKTOP_SPECIAL_FILE_ICON_FILESYSTEM,
+                                                                      TRUE);
+                    xfdesktop_file_icon_manager_set_show_special_file(XFDESKTOP_FILE_ICON_MANAGER(manager),
+                                                                      XFDESKTOP_SPECIAL_FILE_ICON_HOME,
+                                                                      TRUE);
+                    xfdesktop_file_icon_manager_set_show_special_file(XFDESKTOP_FILE_ICON_MANAGER(manager),
+                                                                      XFDESKTOP_SPECIAL_FILE_ICON_TRASH,
+                                                                      TRUE);
                 } else {
                     g_critical("Unable to create ThunarVfsPath for '%s'",
                                desktop_path);
-                    thunar_vfs_shutdown();
                 }
                 g_free(desktop_path);
             }
@@ -218,6 +230,8 @@ set_imgfile_root_property(XfceDesktop *desktop, const gchar *filename,
 {
     gchar property_name[128];
     
+    gdk_error_trap_push();
+    
     g_snprintf(property_name, 128, XFDESKTOP_IMAGE_FILE_FMT, monitor);
     if(filename) {
         gdk_property_change(gdk_screen_get_root_window(desktop->priv->gscreen),
@@ -229,95 +243,16 @@ set_imgfile_root_property(XfceDesktop *desktop, const gchar *filename,
         gdk_property_delete(gdk_screen_get_root_window(desktop->priv->gscreen),
                             gdk_atom_intern(property_name, FALSE));
     }
+    
+    gdk_error_trap_pop();
 }
 
 static void
-backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
+set_real_root_window_pixmap(GdkScreen *gscreen,
+                            GdkPixmap *pmap)
 {
-    XfceDesktop *desktop = XFCE_DESKTOP(user_data);
-    GdkPixbuf *pix;
-    GdkPixmap *pmap = NULL;
-    GdkColormap *cmap;
-    GdkScreen *gscreen;
-    GdkRectangle rect;
-    Pixmap xid;
+    Window xid;
     GdkWindow *groot;
-    gint i, monitor = -1;
-    
-    TRACE("entering");
-    
-    g_return_if_fail(XFCE_IS_DESKTOP(desktop));
-    
-    if(desktop->priv->updates_frozen || !GTK_WIDGET_REALIZED(GTK_WIDGET(desktop)))
-        return;
-    
-    TRACE("really entering");
-    
-    gscreen = desktop->priv->gscreen;
-    cmap = gdk_drawable_get_colormap(GDK_DRAWABLE(GTK_WIDGET(desktop)->window));
-    
-    for(i = 0; i < XFCE_DESKTOP(desktop)->priv->nbackdrops; i++) {
-        if(backdrop == XFCE_DESKTOP(desktop)->priv->backdrops[i]) {
-            monitor = i;
-            break;
-        }
-    }
-    if(monitor == -1)
-        return;
-    
-    /* create/get the composited backdrop pixmap */
-    pix = xfce_backdrop_get_pixbuf(backdrop);
-    if(!pix)
-        return;
-    
-    if(desktop->priv->nbackdrops == 1) {    
-        /* optimised for single monitor: just dump the pixbuf into a pixmap */
-        gdk_pixbuf_render_pixmap_and_mask_for_colormap(pix, cmap, &pmap, NULL, 0);
-        g_object_unref(G_OBJECT(pix));
-        if(!pmap)
-            return;
-        rect.x = rect.y = 0;
-        rect.width = gdk_screen_get_width(gscreen);
-        rect.height = gdk_screen_get_height(gscreen);
-    } else {
-        /* multiple monitors (xinerama): download the current backdrop, paint
-         * over the correct area, and upload it back.  this is slow, but
-         * probably still faster than redoing the whole thing. */
-        GdkPixmap *cur_pmap = NULL;
-        GdkPixbuf *cur_pbuf = NULL;
-        gint swidth, sheight;
-        
-        swidth = gdk_screen_get_width(gscreen);
-        sheight = gdk_screen_get_height(gscreen);
-        
-        cur_pmap = desktop->priv->bg_pixmap;
-        if(cur_pmap) {
-            gint pw, ph;
-            gdk_drawable_get_size(GDK_DRAWABLE(cur_pmap), &pw, &ph);
-            if(pw == swidth && ph == sheight) {
-                cur_pbuf = gdk_pixbuf_get_from_drawable(NULL, 
-                        GDK_DRAWABLE(cur_pmap), cmap, 0, 0, 0, 0, swidth,
-                        sheight);
-            } else
-                cur_pmap = NULL;
-        }
-        /* if the style's bg_pixmap was empty, or the above failed... */
-        if(!cur_pmap) {
-            cur_pbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8,
-                    swidth, sheight);
-        }
-        
-        gdk_screen_get_monitor_geometry(gscreen, monitor, &rect);
-        gdk_pixbuf_copy_area(pix, 0, 0, gdk_pixbuf_get_width(pix),
-                gdk_pixbuf_get_height(pix), cur_pbuf, rect.x, rect.y);
-        g_object_unref(G_OBJECT(pix));
-        pmap = NULL;
-        gdk_pixbuf_render_pixmap_and_mask_for_colormap(cur_pbuf, cmap,
-                &pmap, NULL, 0);
-        g_object_unref(G_OBJECT(cur_pbuf));
-        if(!pmap)
-            return;
-    }
     
     xid = GDK_DRAWABLE_XID(pmap);
     groot = gdk_screen_get_root_window(gscreen);
@@ -338,17 +273,60 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
     gdk_window_set_back_pixmap(groot, pmap, FALSE);
     /* there really should be a standard for this crap... */
     
-    /* clear the old pixmap, if any */
-    if(desktop->priv->bg_pixmap)
-        g_object_unref(G_OBJECT(desktop->priv->bg_pixmap));
+    gdk_error_trap_pop();
+}
+
+static void
+backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
+{
+    XfceDesktop *desktop = XFCE_DESKTOP(user_data);
+    GdkPixmap *pmap = desktop->priv->bg_pixmap;
+    GdkScreen *gscreen = desktop->priv->gscreen;
+    GdkPixbuf *pix;
+    GdkRectangle rect;
+    gint i, monitor = -1;
     
-    /* set the new pixmap and tell gtk to redraw it */
-    desktop->priv->bg_pixmap = pmap;
+    TRACE("entering");
+    
+    g_return_if_fail(XFCE_IS_DESKTOP(desktop));
+    
+    if(desktop->priv->updates_frozen || !GTK_WIDGET_REALIZED(GTK_WIDGET(desktop)))
+        return;
+    
+    TRACE("really entering");
+    
+    for(i = 0; i < XFCE_DESKTOP(desktop)->priv->nbackdrops; i++) {
+        if(backdrop == XFCE_DESKTOP(desktop)->priv->backdrops[i]) {
+            monitor = i;
+            break;
+        }
+    }
+    if(monitor == -1)
+        return;
+    
+    /* create/get the composited backdrop pixmap */
+    pix = xfce_backdrop_get_pixbuf(backdrop);
+    if(!pix)
+        return;
+    
+    if(desktop->priv->nbackdrops == 1) {
+        /* single monitor */
+        rect.x = rect.y = 0;
+        rect.width = gdk_screen_get_width(gscreen);
+        rect.height = gdk_screen_get_height(gscreen);
+    } else
+        gdk_screen_get_monitor_geometry(gscreen, monitor, &rect);
+
+    gdk_draw_pixbuf(GDK_DRAWABLE(pmap), GTK_WIDGET(desktop)->style->black_gc,
+                    pix, 0, 0, rect.x, rect.y,
+                    gdk_pixbuf_get_width(pix), gdk_pixbuf_get_height(pix),
+                    GDK_RGB_DITHER_MAX, 0, 0);
+    g_object_unref(G_OBJECT(pix));
+    
+    /* tell gtk to redraw the repainted area */
     gtk_widget_queue_draw_area(GTK_WIDGET(desktop), rect.x, rect.y,
                                rect.width, rect.height);
     
-    gdk_error_trap_pop();
-    
     set_imgfile_root_property(desktop,
                               xfce_backdrop_get_image_filename(backdrop),
                               monitor);
@@ -368,10 +346,13 @@ screen_size_changed_cb(GdkScreen *gscreen, gpointer user_data)
     gtk_widget_set_size_request(GTK_WIDGET(desktop), w, h);
     gtk_window_resize(GTK_WINDOW(desktop), w, h);
     
-    if(desktop->priv->bg_pixmap) {
-        g_object_unref(G_OBJECT(desktop->priv->bg_pixmap));
-        desktop->priv->bg_pixmap = NULL;
-    }
+    g_object_unref(G_OBJECT(desktop->priv->bg_pixmap));
+    desktop->priv->bg_pixmap = gdk_pixmap_new(GDK_DRAWABLE(GTK_WIDGET(desktop)->window),
+                                              w, h, -1);
+    set_real_root_window_pixmap(desktop->priv->gscreen,
+                                desktop->priv->bg_pixmap);
+    gdk_window_set_back_pixmap(GTK_WIDGET(desktop)->window,
+                               desktop->priv->bg_pixmap, FALSE);
     
     /* special case for 1 backdrop to handle xinerama stretching properly.
      * this is broken if it ever becomes possible to change the number of
@@ -446,8 +427,6 @@ handle_xinerama_unstretch(XfceDesktop *desktop)
     }
 }
 
-
-
 static void
 screen_set_selection(XfceDesktop *desktop)
 {
@@ -503,24 +482,24 @@ G_DEFINE_TYPE(XfceDesktop, xfce_desktop, GTK_TYPE_WINDOW)
 static void
 xfce_desktop_class_init(XfceDesktopClass *klass)
 {
-    GObjectClass *gobject_class;
-    GtkWidgetClass *widget_class;
+    GObjectClass *gobject_class = (GObjectClass *)klass;
+    GtkWidgetClass *widget_class = (GtkWidgetClass *)klass;
     
-    gobject_class = (GObjectClass *)klass;
-    widget_class = (GtkWidgetClass *)klass;
+    g_type_class_add_private(klass, sizeof(XfceDesktopPriv));
     
     gobject_class->finalize = xfce_desktop_finalize;
     
     widget_class->realize = xfce_desktop_realize;
     widget_class->unrealize = xfce_desktop_unrealize;
     widget_class->expose_event = xfce_desktop_expose;
-    widget_class->delete_event = (gboolean (*)(GtkWidget *, GdkEventAny *))gtk_true;
+    widget_class->delete_event = xfce_desktop_delete_event;
 }
 
 static void
 xfce_desktop_init(XfceDesktop *desktop)
 {
-    desktop->priv = g_new0(XfceDesktopPriv, 1);
+    desktop->priv = G_TYPE_INSTANCE_GET_PRIVATE(desktop, XFCE_TYPE_DESKTOP,
+                                                XfceDesktopPriv);
     GTK_WINDOW(desktop)->type = GTK_WINDOW_TOPLEVEL;
 #ifdef ENABLE_DESKTOP_ICONS
     desktop->priv->icons_use_system_font = TRUE;
@@ -528,8 +507,6 @@ xfce_desktop_init(XfceDesktop *desktop)
     
     gtk_window_set_type_hint(GTK_WINDOW(desktop), GDK_WINDOW_TYPE_HINT_DESKTOP);
     gtk_window_set_accept_focus(GTK_WINDOW(desktop), FALSE);
-    
-    gtk_widget_set_app_paintable(GTK_WIDGET(desktop), TRUE);
 }
 
 static void
@@ -539,9 +516,6 @@ xfce_desktop_finalize(GObject *object)
     
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
     
-    g_free(desktop->priv);
-    desktop->priv = NULL;
-    
     G_OBJECT_CLASS(xfce_desktop_parent_class)->finalize(object);
 }
 
@@ -550,7 +524,7 @@ xfce_desktop_realize(GtkWidget *widget)
 {
     XfceDesktop *desktop = XFCE_DESKTOP(widget);
     GdkAtom atom;
-    gint i;
+    gint i, sw, sh;
     Window xid;
     GdkDisplay *gdpy;
     GdkWindow *groot;
@@ -559,6 +533,8 @@ xfce_desktop_realize(GtkWidget *widget)
     TRACE("entering");
     
     gtk_window_set_screen(GTK_WINDOW(desktop), desktop->priv->gscreen);
+    sw = gdk_screen_get_width(desktop->priv->gscreen);
+    sh = gdk_screen_get_height(desktop->priv->gscreen);
     
     /* chain up */
     GTK_WIDGET_CLASS(xfce_desktop_parent_class)->realize(widget);
@@ -567,9 +543,7 @@ xfce_desktop_realize(GtkWidget *widget)
     if(GTK_WIDGET_DOUBLE_BUFFERED(GTK_WIDGET(desktop)))
         gtk_widget_set_double_buffered(GTK_WIDGET(desktop), FALSE);
     
-    gtk_widget_set_size_request(GTK_WIDGET(desktop),
-                      gdk_screen_get_width(desktop->priv->gscreen),
-                      gdk_screen_get_height(desktop->priv->gscreen));
+    gtk_widget_set_size_request(GTK_WIDGET(desktop), sw, sh);
     gtk_window_move(GTK_WINDOW(desktop), 0, 0);
     
     atom = gdk_atom_intern("_NET_WM_WINDOW_TYPE_DESKTOP", FALSE);
@@ -604,6 +578,16 @@ xfce_desktop_realize(GtkWidget *widget)
                 rect.width, rect.height);
     }
     
+    desktop->priv->bg_pixmap = gdk_pixmap_new(GDK_DRAWABLE(GTK_WIDGET(desktop)->window),
+                                              sw, sh, -1);
+    set_real_root_window_pixmap(desktop->priv->gscreen,
+                                desktop->priv->bg_pixmap);
+    gdk_window_set_back_pixmap(GTK_WIDGET(desktop)->window,
+                               desktop->priv->bg_pixmap, FALSE);
+    
+    g_signal_connect(G_OBJECT(desktop), "style-set",
+                     G_CALLBACK(style_set_cb), NULL);
+    
     for(i = 0; i < desktop->priv->nbackdrops; i++) {
         g_signal_connect(G_OBJECT(desktop->priv->backdrops[i]), "changed",
                 G_CALLBACK(backdrop_changed_cb), desktop);
@@ -628,6 +612,9 @@ xfce_desktop_unrealize(GtkWidget *widget)
     
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
     
+    g_signal_handlers_disconnect_by_func(G_OBJECT(desktop),
+                                         G_CALLBACK(style_set_cb), NULL);
+    
     if(GTK_WIDGET_MAPPED(widget))
         gtk_widget_unmap(widget);
     GTK_WIDGET_UNSET_FLAGS(widget, GTK_MAPPED);
@@ -644,6 +631,7 @@ xfce_desktop_unrealize(GtkWidget *widget)
     gdk_property_delete(groot, gdk_atom_intern("NAUTILUS_DESKTOP_WINDOW_ID", FALSE));
     gdk_property_delete(groot, gdk_atom_intern("_XROOTPMAP_ID", FALSE));
     gdk_property_delete(groot, gdk_atom_intern("ESETROOT_PMAP_ID", FALSE));
+    gdk_window_set_back_pixmap(groot, NULL, FALSE);
     
     if(desktop->priv->backdrops) {
         for(i = 0; i < desktop->priv->nbackdrops; i++) {
@@ -655,10 +643,8 @@ xfce_desktop_unrealize(GtkWidget *widget)
         desktop->priv->backdrops = NULL;
     }
     
-    if(desktop->priv->bg_pixmap) {
-        g_object_unref(G_OBJECT(desktop->priv->bg_pixmap));
-        desktop->priv->bg_pixmap = NULL;
-    }
+    g_object_unref(G_OBJECT(desktop->priv->bg_pixmap));
+    desktop->priv->bg_pixmap = NULL;
     
     gtk_window_set_icon(GTK_WINDOW(widget), NULL);
     
@@ -685,16 +671,8 @@ xfce_desktop_expose(GtkWidget *w,
     
     desktop = XFCE_DESKTOP(w);
     
-    if(!desktop->priv->bg_pixmap && desktop->priv->nbackdrops > 0)
-        backdrop_changed_cb(desktop->priv->backdrops[0], desktop);
-    
-    if(desktop->priv->bg_pixmap) {
-        GtkStyle *style = gtk_widget_get_style(w);
-        gdk_draw_drawable(GDK_DRAWABLE(w->window), style->black_gc,
-                          GDK_DRAWABLE(desktop->priv->bg_pixmap),
-                          evt->area.x, evt->area.y, evt->area.x, evt->area.y,
+    gdk_window_clear_area(w->window, evt->area.x, evt->area.y,
                           evt->area.width, evt->area.height);
-    }
     
     children = gtk_container_get_children(GTK_CONTAINER(w));
     for(l = children; l; l = l->next) {
@@ -707,6 +685,27 @@ xfce_desktop_expose(GtkWidget *w,
     return FALSE;
 }
 
+static gboolean
+xfce_desktop_delete_event(GtkWidget *w,
+                          GdkEventAny *evt)
+{
+    if(XFCE_DESKTOP(w)->priv->session_logout_func)
+        XFCE_DESKTOP(w)->priv->session_logout_func();
+    
+    return TRUE;
+}
+
+static void
+style_set_cb(GtkWidget *w,
+             GtkStyle *old_style,
+             gpointer user_data)
+{
+    XfceDesktop *desktop = XFCE_DESKTOP(w);
+    
+    gdk_window_set_back_pixmap(w->window, desktop->priv->bg_pixmap, FALSE);
+    gtk_widget_queue_draw(w);
+}
+
 
 
 /* public api */
@@ -799,6 +798,18 @@ xfce_desktop_set_icon_style(XfceDesktop *desktop,
     if(style == desktop->priv->icons_style)
         return;
     
+    if(desktop->priv->icon_view) {
+        gtk_widget_destroy(desktop->priv->icon_view);
+        desktop->priv->icon_view = NULL;
+    }
+    
+#ifdef HAVE_THUNAR_VFS
+    if(XFCE_DESKTOP_ICON_STYLE_FILES == desktop->priv->icons_style)
+        thunar_vfs_shutdown();
+    else if(XFCE_DESKTOP_ICON_STYLE_FILES == style)
+        thunar_vfs_init();
+#endif
+    
     desktop->priv->icons_style = style;
     xfce_desktop_setup_icon_view(desktop);
 #endif
@@ -882,6 +893,14 @@ xfce_desktop_set_icon_use_system_font_size(XfceDesktop *desktop,
 #endif
 }
 
+void
+xfce_desktop_set_session_logout_func(XfceDesktop *desktop,
+                                     SessionLogoutFunc logout_func)
+{
+    g_return_if_fail(XFCE_IS_DESKTOP(desktop));
+    desktop->priv->session_logout_func = logout_func;
+}
+
 void
 xfce_desktop_freeze_updates(XfceDesktop *desktop)
 {
